import random
import unittest
from tqsdk import TqApi, TqSim, TargetPosTask, TqBacktest, BacktestFinished, utils
from tqsdk.test.api.helper import MockServer, MockInsServer
from tqsdk.tafunc import ma
from contextlib import closing
from datetime import date, datetime
import json
import os


class Unit_test(unittest.TestCase):
    '''
     回测测试example

     注：
    1. 在本地运行测试用例前需设置运行环境变量(Environment variables), 保证api中dict及set等类型的数据序列在每次运行时元素顺序一致: PYTHONHASHSEED=32
    2. 若测试用例中调用了会使用uuid的功能函数时（如insert_order()会使用uuid生成order_id）,
        则：在生成script文件时及测试用例中都需设置 utils.RD = random.Random(x), 以保证两次生成的uuid一致, x取值范围为0-2^32
    3. 对盘中的测试用例（即非回测）：因为TqSim模拟交易 Order 的 insert_date_time 和 Trade 的 trade_date_time 不是固定值，所以改为判断范围。
        盘中时：self.assertAlmostEqual(1575292560005832000 / 1e9, order1.insert_date_time / 1e9, places=1)
        回测时：self.assertEqual(1575291600000000000, order1.insert_date_time)
    '''

    def setUp(self):
        self.ins = MockInsServer(5000)
        os.environ["TQ_INS_URL"] = "http://127.0.0.1:5000/t/md/symbols/2020-08-03.json"
        os.environ["TQ_AUTH_URL"] = "http://127.0.0.1:5000"
        self.mock = MockServer(md_url_character="nfmd")
        self.md_url = "ws://127.0.0.1:5100/"
        self.td_url = "ws://127.0.0.1:5200/"

    def tearDown(self):
        self.ins.close()
        self.mock.close()

    # @unittest.skip("无条件跳过: test_doublema")
    def test_doublema(self):
        '''
        设定：
            回测周期: 2020,6,10-2020,6,15
            合约代码: SHFE.bu2012
        '''
        # 预设服务器端响应
        dir_path = os.path.dirname(os.path.realpath(__file__))
        self.mock.run(os.path.join(dir_path, "examples_log_file", "test_example_doublema.script.lzma"))
        # 测试
        SHORT = 30  # 短周期
        LONG = 60  # 长周期
        SYMBOL = "SHFE.bu2012"  # 合约代码

        utils.RD = random.Random(4)
        sim = TqSim()
        api = TqApi(sim, backtest=TqBacktest(start_dt=datetime(2020, 6, 10), end_dt=datetime(2020, 6, 15)),
                    auth="tianqin,tianqin", _td_url=self.td_url, _md_url=self.md_url)
        data_length = LONG + 2  # k线数据长度
        klines = api.get_kline_serial(SYMBOL, duration_seconds=240, data_length=data_length)
        target_pos = TargetPosTask(api, SYMBOL)

        try:
            with closing(api):
                while True:
                    api.wait_update()
                    if api.is_changing(klines.iloc[-1], "datetime"):  # 产生新k线:重新计算SMA
                        short_avg = ma(klines["close"], SHORT)  # 短周期
                        long_avg = ma(klines["close"], LONG)  # 长周期

                        # 均线下穿，做空
                        if long_avg.iloc[-2] < short_avg.iloc[-2] and long_avg.iloc[-1] > short_avg.iloc[-1]:
                            target_pos.set_target_volume(-3)

                        # 均线上穿，做多
                        if short_avg.iloc[-2] < long_avg.iloc[-2] and short_avg.iloc[-1] > long_avg.iloc[-1]:
                            target_pos.set_target_volume(3)

        except BacktestFinished:
            # 测试
            trade_log = json.dumps(sim.trade_log)
            # print(trade_log)  # 用于生成测试用例except数据
            excepted_trade_log = '{"2020-06-10": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 10000000.0, "available": 10000000.0, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {}}, "2020-06-11": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e", "trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|3", "exchange_trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|3", "exchange_id": "SHFE", "instrument_id": "bu2012", "direction": "BUY", "offset": "OPEN", "price": 2580.0, "volume": 3, "trade_date_time": 1591845120000000000, "commission": 8.472}, {"user_id": "TQSIM", "order_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a", "trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|3", "exchange_trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|3", "exchange_id": "SHFE", "instrument_id": "bu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 2548.0, "volume": 3, "trade_date_time": 1591856640000000000, "commission": 8.472}, {"user_id": "TQSIM", "order_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b", "trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|3", "exchange_trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|3", "exchange_id": "SHFE", "instrument_id": "bu2012", "direction": "SELL", "offset": "OPEN", "price": 2548.0, "volume": 3, "trade_date_time": 1591856640000000000, "commission": 8.472}], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 9998834.584000003, "available": 9991209.784000002, "float_profit": -180.0, "position_profit": -180.0, "close_profit": -960.0, "frozen_margin": 0.0, "margin": 7624.799999999999, "frozen_commission": 0.0, "commission": 25.415999999999997, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0007625688709963359, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.bu2012": {"exchange_id": "SHFE", "instrument_id": "bu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 3, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 3, "volume_short_his": 0, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 2548.0, "open_cost_long": 0.0, "open_cost_short": 76440.0, "position_price_long": NaN, "position_price_short": 2548.0, "position_cost_long": 0.0, "position_cost_short": 76440.0, "float_profit_long": 0.0, "float_profit_short": -180.0, "float_profit": -180.0, "position_profit_long": 0.0, "position_profit_short": -180.0, "position_profit": -180.0, "margin_long": 0.0, "margin_short": 7624.799999999999, "margin": 7624.799999999999, "last_price": 2554.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-12": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad", "trade_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad|3", "exchange_trade_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad|3", "exchange_id": "SHFE", "instrument_id": "bu2012", "direction": "BUY", "offset": "CLOSE", "price": 2532.0, "volume": 3, "trade_date_time": 1591931760000000000, "commission": 8.472}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2a3187853184ff27459142deccea2645", "trade_id": "PYSDK_target_2a3187853184ff27459142deccea2645|3", "exchange_trade_id": "PYSDK_target_2a3187853184ff27459142deccea2645|3", "exchange_id": "SHFE", "instrument_id": "bu2012", "direction": "BUY", "offset": "OPEN", "price": 2532.0, "volume": 3, "trade_date_time": 1591931760000000000, "commission": 8.472}], "account": {"currency": "CNY", "pre_balance": 9998834.584000003, "static_balance": 9998834.584000003, "balance": 9999297.640000004, "available": 9991672.840000004, "float_profit": -180.0, "position_profit": -180.0, "close_profit": 660.0, "frozen_margin": 0.0, "margin": 7624.799999999999, "frozen_commission": 0.0, "commission": 16.944, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0007625335573069306, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.bu2012": {"exchange_id": "SHFE", "instrument_id": "bu2012", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 2532.0, "open_price_short": NaN, "open_cost_long": 75960.0, "open_cost_short": 0.0, "position_price_long": 2532.0, "position_price_short": NaN, "position_cost_long": 75960.0, "position_cost_short": 0.0, "float_profit_long": -180.0, "float_profit_short": 0.0, "float_profit": -180.0, "position_profit_long": -180.0, "position_profit_short": 0.0, "position_profit": -180.0, "margin_long": 7624.799999999999, "margin_short": 0.0, "margin": 7624.799999999999, "last_price": 2526.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-15": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9999297.640000004, "static_balance": 9999297.640000004, "balance": 9999897.640000004, "available": 9992272.840000004, "float_profit": 420.0, "position_profit": 600.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 7624.799999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0007624878048251698, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.bu2012": {"exchange_id": "SHFE", "instrument_id": "bu2012", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 2532.0, "open_price_short": NaN, "open_cost_long": 75960.0, "open_cost_short": 0.0, "position_price_long": 2526.0, "position_price_short": 2526.0, "position_cost_long": 75780.0, "position_cost_short": 0.0, "float_profit_long": 420.0, "float_profit_short": 0.0, "float_profit": 420.0, "position_profit_long": 600.0, "position_profit_short": 0.0, "position_profit": 600.0, "margin_long": 7624.799999999999, "margin_short": 0.0, "margin": 7624.799999999999, "last_price": 2546.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}}'
            self.assertEqual(trade_log, excepted_trade_log, msg="交易记录及收盘时的权益及持仓: 信息错误")

    # @unittest.skip("无条件跳过: test_dualthrust")
    def test_dualthrust(self):
        '''
        设定：
            回测周期: 2019,5,1-2019,6,10
            合约代码: DCE.jd1909
            合约文件: "http://127.0.0.1:5000/t/md/symbols/2019-07-03.json"
        '''
        # 预设服务器端响应
        dir_path = os.path.dirname(os.path.realpath(__file__))
        self.mock.run(os.path.join(dir_path, "examples_log_file", "test_example_dualthrust.script.lzma"))
        # 测试
        SYMBOL = "DCE.jd2009"  # 合约代码
        NDAY = 5  # 天数
        K1 = 0.2  # 上轨K值
        K2 = 0.2  # 下轨K值
        utils.RD = random.Random(4)
        sim = TqSim()
        api = TqApi(sim, backtest=TqBacktest(start_dt=date(2020, 6, 1), end_dt=date(2020, 7, 10)),
                    auth="tianqin,tianqin", _td_url=self.td_url, _md_url=self.md_url)

        quote = api.get_quote(SYMBOL)
        klines = api.get_kline_serial(SYMBOL, 24 * 60 * 60)  # 86400使用日线
        target_pos = TargetPosTask(api, SYMBOL)

        def dual_thrust(quote, klines):
            current_open = klines.iloc[-1]["open"]
            HH = max(klines.high.iloc[-NDAY - 1:-1])  # N日最高价的最高价
            HC = max(klines.close.iloc[-NDAY - 1:-1])  # N日收盘价的最高价
            LC = min(klines.close.iloc[-NDAY - 1:-1])  # N日收盘价的最低价
            LL = min(klines.low.iloc[-NDAY - 1:-1])  # N日最低价的最低价
            range = max(HH - LC, HC - LL)
            buy_line = current_open + range * K1  # 上轨
            sell_line = current_open - range * K2  # 下轨
            return buy_line, sell_line

        buy_line, sell_line = dual_thrust(quote, klines)  # 获取上下轨

        try:
            with closing(api):
                while True:
                    api.wait_update()
                    if api.is_changing(klines.iloc[-1], ["datetime", "open"]):  # 新产生一根日线或开盘价发生变化: 重新计算上下轨
                        buy_line, sell_line = dual_thrust(quote, klines)

                    if api.is_changing(quote, "last_price"):
                        if quote.last_price > buy_line:  # 高于上轨
                            target_pos.set_target_volume(3)  # 交易
                        elif quote.last_price < sell_line:  # 低于下轨
                            target_pos.set_target_volume(-3)  # 交易
                        else:
                            pass

        except BacktestFinished:
            # 测试
            trade_log = json.dumps(sim.trade_log)
            # print(trade_log)  # 用于生成测试用例except数据
            excepted_trade_log = '{"2020-06-01": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e", "trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|3", "exchange_trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 4251.0, "volume": 3, "trade_date_time": 1590973499999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a", "trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|3", "exchange_trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "CLOSE", "price": 4201.0, "volume": 3, "trade_date_time": 1590977279999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b", "trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|3", "exchange_trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "OPEN", "price": 4201.0, "volume": 3, "trade_date_time": 1590977279999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 9998744.974, "available": 9990185.374, "float_profit": 300.0, "position_profit": 300.0, "close_profit": -1500.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 55.025999999999996, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008560674386893307, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 3, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 3, "volume_short_his": 0, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4201.0, "open_cost_long": 0.0, "open_cost_short": 126030.0, "position_price_long": NaN, "position_price_short": 4201.0, "position_cost_long": 0.0, "position_cost_short": 126030.0, "float_profit_long": 0.0, "float_profit_short": 300.0, "float_profit": 300.0, "position_profit_long": 0.0, "position_profit_short": 300.0, "position_profit": 300.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 4191.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-02": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9998744.974, "static_balance": 9998744.974, "balance": 10001954.974, "available": 9993395.374, "float_profit": 3510.0, "position_profit": 3210.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008557926947532367, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4201.0, "open_cost_long": 0.0, "open_cost_short": 126030.0, "position_price_long": 4191.0, "position_price_short": 4191.0, "position_cost_long": 0.0, "position_cost_short": 126330.0, "float_profit_long": 0.0, "float_profit_short": 3510.0, "float_profit": 3510.0, "position_profit_long": 0.0, "position_profit_short": 3210.0, "position_profit": 3210.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 4084.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-03": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001954.974, "static_balance": 10001954.974, "balance": 10001534.974, "available": 9992975.374, "float_profit": 3090.0, "position_profit": -420.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000855828632530061, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4201.0, "open_cost_long": 0.0, "open_cost_short": 126030.0, "position_price_long": 4084.0, "position_price_short": 4084.0, "position_cost_long": 0.0, "position_cost_short": 129540.0, "float_profit_long": 0.0, "float_profit_short": 3090.0, "float_profit": 3090.0, "position_profit_long": 0.0, "position_profit_short": -420.0, "position_profit": -420.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 4098.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-04": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001534.974, "static_balance": 10001534.974, "balance": 10002254.974, "available": 9993695.374, "float_profit": 3810.0, "position_profit": 720.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008557670267604596, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4201.0, "open_cost_long": 0.0, "open_cost_short": 126030.0, "position_price_long": 4098.0, "position_price_short": 4098.0, "position_cost_long": 0.0, "position_cost_short": 129120.0, "float_profit_long": 0.0, "float_profit_short": 3810.0, "float_profit": 3810.0, "position_profit_long": 0.0, "position_profit_short": 720.0, "position_profit": 720.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 4074.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-05": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad", "trade_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad|3", "exchange_trade_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "CLOSE", "price": 4098.0, "volume": 3, "trade_date_time": 1591327679999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2a3187853184ff27459142deccea2645", "trade_id": "PYSDK_target_2a3187853184ff27459142deccea2645|3", "exchange_trade_id": "PYSDK_target_2a3187853184ff27459142deccea2645|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 4098.0, "volume": 3, "trade_date_time": 1591327679999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10002254.974, "static_balance": 10002254.974, "balance": 10001048.29, "available": 9992488.69, "float_profit": -450.0, "position_profit": -450.0, "close_profit": -720.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008558702799744205, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 4098.0, "open_price_short": NaN, "open_cost_long": 122940.0, "open_cost_short": 0.0, "position_price_long": 4098.0, "position_price_short": NaN, "position_cost_long": 122940.0, "position_cost_short": 0.0, "float_profit_long": -450.0, "float_profit_short": 0.0, "float_profit": -450.0, "position_profit_long": -450.0, "position_profit_short": 0.0, "position_profit": -450.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 4083.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-08": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001048.29, "static_balance": 10001048.29, "balance": 10000658.29, "available": 9992098.69, "float_profit": -840.0, "position_profit": -390.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008559036567181819, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 4098.0, "open_price_short": NaN, "open_cost_long": 122940.0, "open_cost_short": 0.0, "position_price_long": 4083.0, "position_price_short": 4083.0, "position_cost_long": 122490.0, "position_cost_short": 0.0, "float_profit_long": -840.0, "float_profit_short": 0.0, "float_profit": -840.0, "position_profit_long": -390.0, "position_profit_short": 0.0, "position_profit": -390.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 4070.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-09": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_de08caa1a08179104a25e4664f5253a0", "trade_id": "PYSDK_target_de08caa1a08179104a25e4664f5253a0|3", "exchange_trade_id": "PYSDK_target_de08caa1a08179104a25e4664f5253a0|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "CLOSE", "price": 4035.0, "volume": 3, "trade_date_time": 1591680779999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d93936e1daca3c06f5ff0c03bb5d7385", "trade_id": "PYSDK_target_d93936e1daca3c06f5ff0c03bb5d7385|3", "exchange_trade_id": "PYSDK_target_d93936e1daca3c06f5ff0c03bb5d7385|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "OPEN", "price": 4035.0, "volume": 3, "trade_date_time": 1591680779999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d8441b5616332aca5f552773e14b0190", "trade_id": "PYSDK_target_d8441b5616332aca5f552773e14b0190|3", "exchange_trade_id": "PYSDK_target_d8441b5616332aca5f552773e14b0190|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "CLOSE", "price": 4085.0, "volume": 3, "trade_date_time": 1591683479999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_634f806fabf4a07c566002249b191bf4", "trade_id": "PYSDK_target_634f806fabf4a07c566002249b191bf4|3", "exchange_trade_id": "PYSDK_target_634f806fabf4a07c566002249b191bf4|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 4085.0, "volume": 3, "trade_date_time": 1591683479999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10000658.29, "static_balance": 10000658.29, "balance": 9997854.921999998, "available": 9989295.321999999, "float_profit": -180.0, "position_profit": -180.0, "close_profit": -2550.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 73.368, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008561436494907362, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 4085.0, "open_price_short": NaN, "open_cost_long": 122550.0, "open_cost_short": 0.0, "position_price_long": 4085.0, "position_price_short": NaN, "position_cost_long": 122550.0, "position_cost_short": 0.0, "float_profit_long": -180.0, "float_profit_short": 0.0, "float_profit": -180.0, "position_profit_long": -180.0, "position_profit_short": 0.0, "position_profit": -180.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 4079.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-10": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9997854.921999998, "static_balance": 9997854.921999998, "balance": 9997494.921999998, "available": 9988935.321999999, "float_profit": -540.0, "position_profit": -360.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008561744783849963, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 4085.0, "open_price_short": NaN, "open_cost_long": 122550.0, "open_cost_short": 0.0, "position_price_long": 4079.0, "position_price_short": 4079.0, "position_cost_long": 122370.0, "position_cost_short": 0.0, "float_profit_long": -540.0, "float_profit_short": 0.0, "float_profit": -540.0, "position_profit_long": -360.0, "position_profit_short": 0.0, "position_profit": -360.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 4067.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-11": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_3f5082492d83a8233fb62d2c81862fc9", "trade_id": "PYSDK_target_3f5082492d83a8233fb62d2c81862fc9|3", "exchange_trade_id": "PYSDK_target_3f5082492d83a8233fb62d2c81862fc9|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "CLOSE", "price": 4042.0, "volume": 3, "trade_date_time": 1591837919999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f1cfd99216df648647adec26793d0e45", "trade_id": "PYSDK_target_f1cfd99216df648647adec26793d0e45|3", "exchange_trade_id": "PYSDK_target_f1cfd99216df648647adec26793d0e45|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "OPEN", "price": 4042.0, "volume": 3, "trade_date_time": 1591837919999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 9997494.921999998, "static_balance": 9997494.921999998, "balance": 10001688.237999998, "available": 9993128.637999998, "float_profit": 4980.0, "position_profit": 4980.0, "close_profit": -750.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000855815517972157, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 3, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 3, "volume_short_his": 0, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4042.0, "open_cost_long": 0.0, "open_cost_short": 121260.0, "position_price_long": NaN, "position_price_short": 4042.0, "position_cost_long": 0.0, "position_cost_short": 121260.0, "float_profit_long": 0.0, "float_profit_short": 4980.0, "float_profit": 4980.0, "position_profit_long": 0.0, "position_profit_short": 4980.0, "position_profit": 4980.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3876.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-12": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001688.237999998, "static_balance": 10001688.237999998, "balance": 10003428.237999998, "available": 9994868.637999998, "float_profit": 6720.0, "position_profit": 1740.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000855666657105078, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4042.0, "open_cost_long": 0.0, "open_cost_short": 121260.0, "position_price_long": 3876.0, "position_price_short": 3876.0, "position_cost_long": 0.0, "position_cost_short": 126240.0, "float_profit_long": 0.0, "float_profit_short": 6720.0, "float_profit": 6720.0, "position_profit_long": 0.0, "position_profit_short": 1740.0, "position_profit": 1740.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3818.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-15": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10003428.237999998, "static_balance": 10003428.237999998, "balance": 10005258.237999998, "available": 9996698.637999998, "float_profit": 8550.0, "position_profit": 1830.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008555101524007261, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4042.0, "open_cost_long": 0.0, "open_cost_short": 121260.0, "position_price_long": 3818.0, "position_price_short": 3818.0, "position_cost_long": 0.0, "position_cost_short": 127980.0, "float_profit_long": 0.0, "float_profit_short": 8550.0, "float_profit": 8550.0, "position_profit_long": 0.0, "position_profit_short": 1830.0, "position_profit": 1830.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3757.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-16": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10005258.237999998, "static_balance": 10005258.237999998, "balance": 10005348.237999998, "available": 9996788.637999998, "float_profit": 8640.0, "position_profit": 90.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000855502456925078, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4042.0, "open_cost_long": 0.0, "open_cost_short": 121260.0, "position_price_long": 3757.0, "position_price_short": 3757.0, "position_cost_long": 0.0, "position_cost_short": 129810.0, "float_profit_long": 0.0, "float_profit_short": 8640.0, "float_profit": 8640.0, "position_profit_long": 0.0, "position_profit_short": 90.0, "position_profit": 90.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3754.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-17": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10005348.237999998, "static_balance": 10005348.237999998, "balance": 10006938.237999998, "available": 9998378.637999998, "float_profit": 10230.0, "position_profit": 1590.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008553665263462976, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 4042.0, "open_cost_long": 0.0, "open_cost_short": 121260.0, "position_price_long": 3754.0, "position_price_short": 3754.0, "position_cost_long": 0.0, "position_cost_short": 129900.0, "float_profit_long": 0.0, "float_profit_short": 10230.0, "float_profit": 10230.0, "position_profit_long": 0.0, "position_profit_short": 1590.0, "position_profit": 1590.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3701.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-18": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_f1347e0cdd905ecfd160c5d0ef412ed6", "trade_id": "PYSDK_target_f1347e0cdd905ecfd160c5d0ef412ed6|3", "exchange_trade_id": "PYSDK_target_f1347e0cdd905ecfd160c5d0ef412ed6|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "CLOSE", "price": 3832.0, "volume": 3, "trade_date_time": 1592450279999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_01d89a024cdce7a6d7288ff68c320f89", "trade_id": "PYSDK_target_01d89a024cdce7a6d7288ff68c320f89|3", "exchange_trade_id": "PYSDK_target_01d89a024cdce7a6d7288ff68c320f89|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 3832.0, "volume": 3, "trade_date_time": 1592450279999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10006938.237999998, "static_balance": 10006938.237999998, "balance": 10003721.553999998, "available": 9995161.953999998, "float_profit": 750.0, "position_profit": 750.0, "close_profit": -3930.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008556415683698668, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3832.0, "open_price_short": NaN, "open_cost_long": 114960.0, "open_cost_short": 0.0, "position_price_long": 3832.0, "position_price_short": NaN, "position_cost_long": 114960.0, "position_cost_short": 0.0, "float_profit_long": 750.0, "float_profit_short": 0.0, "float_profit": 750.0, "position_profit_long": 750.0, "position_profit_short": 0.0, "position_profit": 750.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3857.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-19": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10003721.553999998, "static_balance": 10003721.553999998, "balance": 10004231.553999998, "available": 9995671.953999998, "float_profit": 1260.0, "position_profit": 510.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008555979491076062, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3832.0, "open_price_short": NaN, "open_cost_long": 114960.0, "open_cost_short": 0.0, "position_price_long": 3857.0, "position_price_short": 3857.0, "position_cost_long": 115710.0, "position_cost_short": 0.0, "float_profit_long": 1260.0, "float_profit_short": 0.0, "float_profit": 1260.0, "position_profit_long": 510.0, "position_profit_short": 0.0, "position_profit": 510.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3874.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-22": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_b474c7e89286a1754abcb06ae8abb93f", "trade_id": "PYSDK_target_b474c7e89286a1754abcb06ae8abb93f|3", "exchange_trade_id": "PYSDK_target_b474c7e89286a1754abcb06ae8abb93f|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "CLOSE", "price": 3854.0, "volume": 3, "trade_date_time": 1592806139999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_c3e4a892d9196ada4fcfa583e1df8af9", "trade_id": "PYSDK_target_c3e4a892d9196ada4fcfa583e1df8af9|3", "exchange_trade_id": "PYSDK_target_c3e4a892d9196ada4fcfa583e1df8af9|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "OPEN", "price": 3854.0, "volume": 3, "trade_date_time": 1592806139999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10004231.553999998, "static_balance": 10004231.553999998, "balance": 10003384.869999997, "available": 9994825.269999998, "float_profit": -210.0, "position_profit": -210.0, "close_profit": -600.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008556703667045853, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 3, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 3, "volume_short_his": 0, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 3854.0, "open_cost_long": 0.0, "open_cost_short": 115620.0, "position_price_long": NaN, "position_price_short": 3854.0, "position_cost_long": 0.0, "position_cost_short": 115620.0, "float_profit_long": 0.0, "float_profit_short": -210.0, "float_profit": -210.0, "position_profit_long": 0.0, "position_profit_short": -210.0, "position_profit": -210.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3861.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-23": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_6c79a3de69f85e3131f3b9238224b122", "trade_id": "PYSDK_target_6c79a3de69f85e3131f3b9238224b122|3", "exchange_trade_id": "PYSDK_target_6c79a3de69f85e3131f3b9238224b122|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "CLOSE", "price": 3920.0, "volume": 3, "trade_date_time": 1592879459999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_738d243a6e58d5ca49c7b59b995253fd", "trade_id": "PYSDK_target_738d243a6e58d5ca49c7b59b995253fd|3", "exchange_trade_id": "PYSDK_target_738d243a6e58d5ca49c7b59b995253fd|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 3920.0, "volume": 3, "trade_date_time": 1592879459999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10003384.869999997, "static_balance": 10003384.869999997, "balance": 10001548.185999997, "available": 9992988.585999997, "float_profit": -30.0, "position_profit": -30.0, "close_profit": -1770.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008558275019843014, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3920.0, "open_price_short": NaN, "open_cost_long": 117600.0, "open_cost_short": 0.0, "position_price_long": 3920.0, "position_price_short": NaN, "position_cost_long": 117600.0, "position_cost_short": 0.0, "float_profit_long": -30.0, "float_profit_short": 0.0, "float_profit": -30.0, "position_profit_long": -30.0, "position_profit_short": 0.0, "position_profit": -30.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3919.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-24": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001548.185999997, "static_balance": 10001548.185999997, "balance": 10000978.185999997, "available": 9992418.585999997, "float_profit": -600.0, "position_profit": -570.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008558762793805779, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3920.0, "open_price_short": NaN, "open_cost_long": 117600.0, "open_cost_short": 0.0, "position_price_long": 3919.0, "position_price_short": 3919.0, "position_cost_long": 117570.0, "position_cost_short": 0.0, "float_profit_long": -600.0, "float_profit_short": 0.0, "float_profit": -600.0, "position_profit_long": -570.0, "position_profit_short": 0.0, "position_profit": -570.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3900.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-29": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000978.185999997, "static_balance": 10000978.185999997, "balance": 10000798.185999997, "available": 9992238.585999997, "float_profit": -780.0, "position_profit": -180.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008558916839240376, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3920.0, "open_price_short": NaN, "open_cost_long": 117600.0, "open_cost_short": 0.0, "position_price_long": 3900.0, "position_price_short": 3900.0, "position_cost_long": 117000.0, "position_cost_short": 0.0, "float_profit_long": -780.0, "float_profit_short": 0.0, "float_profit": -780.0, "position_profit_long": -180.0, "position_profit_short": 0.0, "position_profit": -180.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3894.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-30": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000798.185999997, "static_balance": 10000798.185999997, "balance": 10001248.185999997, "available": 9992688.585999997, "float_profit": -330.0, "position_profit": 450.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008558531736050652, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3920.0, "open_price_short": NaN, "open_cost_long": 117600.0, "open_cost_short": 0.0, "position_price_long": 3894.0, "position_price_short": 3894.0, "position_cost_long": 116820.0, "position_cost_short": 0.0, "float_profit_long": -330.0, "float_profit_short": 0.0, "float_profit": -330.0, "position_profit_long": 450.0, "position_profit_short": 0.0, "position_profit": 450.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3909.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-01": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001248.185999997, "static_balance": 10001248.185999997, "balance": 10002958.185999997, "available": 9994398.585999997, "float_profit": 1380.0, "position_profit": 1710.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008557068659928917, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3920.0, "open_price_short": NaN, "open_cost_long": 117600.0, "open_cost_short": 0.0, "position_price_long": 3909.0, "position_price_short": 3909.0, "position_cost_long": 117270.0, "position_cost_short": 0.0, "float_profit_long": 1380.0, "float_profit_short": 0.0, "float_profit": 1380.0, "position_profit_long": 1710.0, "position_profit_short": 0.0, "position_profit": 1710.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3966.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-02": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10002958.185999997, "static_balance": 10002958.185999997, "balance": 10002718.185999997, "available": 9994158.585999997, "float_profit": 1140.0, "position_profit": -240.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008557273973768635, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3920.0, "open_price_short": NaN, "open_cost_long": 117600.0, "open_cost_short": 0.0, "position_price_long": 3966.0, "position_price_short": 3966.0, "position_cost_long": 118980.0, "position_cost_short": 0.0, "float_profit_long": 1140.0, "float_profit_short": 0.0, "float_profit": 1140.0, "position_profit_long": -240.0, "position_profit_short": 0.0, "position_profit": -240.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3958.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-03": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_4278c2614e1bcb383bb4a570294c4ea3", "trade_id": "PYSDK_target_4278c2614e1bcb383bb4a570294c4ea3|3", "exchange_trade_id": "PYSDK_target_4278c2614e1bcb383bb4a570294c4ea3|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "CLOSE", "price": 3935.0, "volume": 3, "trade_date_time": 1593755699999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_14c15c910b11ad28cc21ce88d0060cc5", "trade_id": "PYSDK_target_14c15c910b11ad28cc21ce88d0060cc5|3", "exchange_trade_id": "PYSDK_target_14c15c910b11ad28cc21ce88d0060cc5|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "OPEN", "price": 3935.0, "volume": 3, "trade_date_time": 1593755699999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10002718.185999997, "static_balance": 10002718.185999997, "balance": 10002141.501999997, "available": 9993581.901999997, "float_profit": 150.0, "position_profit": 150.0, "close_profit": -690.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008557767352409929, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 3, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 3, "volume_short_his": 0, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 3935.0, "open_cost_long": 0.0, "open_cost_short": 118050.0, "position_price_long": NaN, "position_price_short": 3935.0, "position_cost_long": 0.0, "position_cost_short": 118050.0, "float_profit_long": 0.0, "float_profit_short": 150.0, "float_profit": 150.0, "position_profit_long": 0.0, "position_profit_short": 150.0, "position_profit": 150.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3930.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-06": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_ff5a52f1a05885ac7671863c0bdbc23a", "trade_id": "PYSDK_target_ff5a52f1a05885ac7671863c0bdbc23a|3", "exchange_trade_id": "PYSDK_target_ff5a52f1a05885ac7671863c0bdbc23a|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "CLOSE", "price": 3954.0, "volume": 3, "trade_date_time": 1593997859999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_a5e333cb88dcf94384d4cd1f47ca7883", "trade_id": "PYSDK_target_a5e333cb88dcf94384d4cd1f47ca7883|3", "exchange_trade_id": "PYSDK_target_a5e333cb88dcf94384d4cd1f47ca7883|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 3954.0, "volume": 3, "trade_date_time": 1593997859999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10002141.501999997, "static_balance": 10002141.501999997, "balance": 10001204.817999996, "available": 9992645.217999997, "float_profit": -180.0, "position_profit": -180.0, "close_profit": -720.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008558568848219745, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3954.0, "open_price_short": NaN, "open_cost_long": 118620.0, "open_cost_short": 0.0, "position_price_long": 3954.0, "position_price_short": NaN, "position_cost_long": 118620.0, "position_cost_short": 0.0, "float_profit_long": -180.0, "float_profit_short": 0.0, "float_profit": -180.0, "position_profit_long": -180.0, "position_profit_short": 0.0, "position_profit": -180.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3948.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-07": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10001204.817999996, "static_balance": 10001204.817999996, "balance": 10002404.817999996, "available": 9993845.217999997, "float_profit": 1020.0, "position_profit": 1200.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008557542066880183, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 3, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 3, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3954.0, "open_price_short": NaN, "open_cost_long": 118620.0, "open_cost_short": 0.0, "position_price_long": 3948.0, "position_price_short": 3948.0, "position_cost_long": 118440.0, "position_cost_short": 0.0, "float_profit_long": 1020.0, "float_profit_short": 0.0, "float_profit": 1020.0, "position_profit_long": 1200.0, "position_profit_short": 0.0, "position_profit": 1200.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3988.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-08": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_2522d53857c49391b36cc9aa78a330a1", "trade_id": "PYSDK_target_2522d53857c49391b36cc9aa78a330a1|3", "exchange_trade_id": "PYSDK_target_2522d53857c49391b36cc9aa78a330a1|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "CLOSE", "price": 3981.0, "volume": 3, "trade_date_time": 1594170599999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_11021c9e32111ac1ac7cc4a4ff4dab10", "trade_id": "PYSDK_target_11021c9e32111ac1ac7cc4a4ff4dab10|3", "exchange_trade_id": "PYSDK_target_11021c9e32111ac1ac7cc4a4ff4dab10|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "SELL", "offset": "OPEN", "price": 3981.0, "volume": 3, "trade_date_time": 1594170599999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10002404.817999996, "static_balance": 10002404.817999996, "balance": 10002068.133999996, "available": 9993508.533999996, "float_profit": -90.0, "position_profit": -90.0, "close_profit": -210.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000855783012605501, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 3, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 3, "volume_short_his": 0, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 3981.0, "open_cost_long": 0.0, "open_cost_short": 119430.0, "position_price_long": NaN, "position_price_short": 3981.0, "position_cost_long": 0.0, "position_cost_short": 119430.0, "float_profit_long": 0.0, "float_profit_short": -90.0, "float_profit": -90.0, "position_profit_long": 0.0, "position_profit_short": -90.0, "position_profit": -90.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3984.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-09": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10002068.133999996, "static_balance": 10002068.133999996, "balance": 10004648.133999996, "available": 9996088.533999996, "float_profit": 2490.0, "position_profit": 2580.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008555623231676567, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 3, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 3, "volume_short": 3, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 3981.0, "open_cost_long": 0.0, "open_cost_short": 119430.0, "position_price_long": 3984.0, "position_price_short": 3984.0, "position_cost_long": 0.0, "position_cost_short": 119340.0, "float_profit_long": 0.0, "float_profit_short": 2490.0, "float_profit": 2490.0, "position_profit_long": 0.0, "position_profit_short": 2580.0, "position_profit": 2580.0, "margin_long": 0.0, "margin_short": 8559.599999999999, "margin": 8559.599999999999, "last_price": 3898.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-10": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_a2909cb633e238b4e9dd38b869ace913", "trade_id": "PYSDK_target_a2909cb633e238b4e9dd38b869ace913|3", "exchange_trade_id": "PYSDK_target_a2909cb633e238b4e9dd38b869ace913|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "CLOSE", "price": 3911.0, "volume": 3, "trade_date_time": 1594343219999999000, "commission": 18.342}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2f0733c846bbe9e870ef55b1a1f65507", "trade_id": "PYSDK_target_2f0733c846bbe9e870ef55b1a1f65507|3", "exchange_trade_id": "PYSDK_target_2f0733c846bbe9e870ef55b1a1f65507|3", "exchange_id": "DCE", "instrument_id": "jd2009", "direction": "BUY", "offset": "OPEN", "price": 3911.0, "volume": 3, "trade_date_time": 1594343219999999000, "commission": 18.342}], "account": {"currency": "CNY", "pre_balance": 10004648.133999996, "static_balance": 10004648.133999996, "balance": 10003921.449999996, "available": 9995361.849999996, "float_profit": -300.0, "position_profit": -300.0, "close_profit": -390.0, "frozen_margin": 0.0, "margin": 8559.599999999999, "frozen_commission": 0.0, "commission": 36.684, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008556244711417643, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"DCE.jd2009": {"exchange_id": "DCE", "instrument_id": "jd2009", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 3911.0, "open_price_short": NaN, "open_cost_long": 117330.0, "open_cost_short": 0.0, "position_price_long": 3911.0, "position_price_short": NaN, "position_cost_long": 117330.0, "position_cost_short": 0.0, "float_profit_long": -300.0, "float_profit_short": 0.0, "float_profit": -300.0, "position_profit_long": -300.0, "position_profit_short": 0.0, "position_profit": -300.0, "margin_long": 8559.599999999999, "margin_short": 0.0, "margin": 8559.599999999999, "last_price": 3901.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}}'
            self.assertEqual(excepted_trade_log, trade_log)

    # @unittest.skip("无条件跳过: test_rbreaker")
    def test_rbreaker(self):
        '''
        R-Breaker策略(隔夜留仓)
        设定：
            回测周期: 2019,5,1-2019,5,20
            合约代码: SHFE.au1912
            合约文件: "http://127.0.0.1:5000/t/md/symbols/2019-06-05.json"
        '''
        # 预设服务器端响应
        dir_path = os.path.dirname(os.path.realpath(__file__))
        self.mock.run(os.path.join(dir_path, "examples_log_file", "test_example_rbreaker.script.lzma"))
        # 测试
        SYMBOL = "SHFE.au2012"  # 合约代码
        STOP_LOSS_PRICE = 10  # 止损点(价格)

        def get_index_line(klines):
            '''计算指标线'''
            high = klines.high.iloc[-2]  # 前一日的最高价
            low = klines.low.iloc[-2]  # 前一日的最低价
            close = klines.close.iloc[-2]  # 前一日的收盘价

            pivot = (high + low + close) / 3  # 枢轴点
            b_break = high + 2 * (pivot - low)  # 突破买入价
            s_setup = pivot + (high - low)  # 观察卖出价
            s_enter = 2 * pivot - low  # 反转卖出价
            b_enter = 2 * pivot - high  # 反转买入价
            b_setup = pivot - (high - low)  # 观察买入价
            s_break = low - 2 * (high - pivot)  # 突破卖出价

            return pivot, b_break, s_setup, s_enter, b_enter, b_setup, s_break

        utils.RD = random.Random(4)
        sim = TqSim()
        api = TqApi(sim, backtest=TqBacktest(start_dt=date(2020, 6, 1), end_dt=date(2020, 7, 20)),
                    auth="tianqin,tianqin", _td_url=self.td_url, _md_url=self.md_url)
        quote = api.get_quote(SYMBOL)
        klines = api.get_kline_serial(SYMBOL, 24 * 60 * 60)  # 86400: 使用日线
        position = api.get_position(SYMBOL)
        target_pos = TargetPosTask(api, SYMBOL)
        target_pos_value = position.volume_long - position.volume_short  # 目标净持仓数
        open_position_price = position.open_price_long if target_pos_value > 0 else position.open_price_short  # 开仓价
        pivot, b_break, s_setup, s_enter, b_enter, b_setup, s_break = get_index_line(klines)  # 七条标准线
        try:
            with closing(api):
                while True:
                    target_pos.set_target_volume(target_pos_value)

                    api.wait_update()
                    if api.is_changing(klines.iloc[-1], "datetime"):  # 产生新k线,则重新计算7条指标线
                        pivot, b_break, s_setup, s_enter, b_enter, b_setup, s_break = get_index_line(klines)
                    '''交易规则'''
                    if api.is_changing(quote, "last_price"):
                        # 开仓价与当前行情价之差大于止损点则止损
                        if (target_pos_value > 0 and open_position_price - quote.last_price >= STOP_LOSS_PRICE) or \
                                (target_pos_value < 0 and quote.last_price - open_position_price >= STOP_LOSS_PRICE):
                            target_pos_value = 0  # 平仓
                        # 反转:
                        if target_pos_value > 0:  # 多头持仓
                            if quote.highest > s_setup and quote.last_price < s_enter:
                                target_pos_value = -1  # 做空
                                open_position_price = quote.last_price
                        elif target_pos_value < 0:  # 空头持仓
                            if quote.lowest < b_setup and quote.last_price > b_enter:
                                target_pos_value = 1  # 做多
                                open_position_price = quote.last_price

                        # 突破:
                        elif target_pos_value == 0:  # 空仓条件
                            if quote.last_price > b_break:
                                target_pos_value = 1  # 做多
                                open_position_price = quote.last_price
                            elif quote.last_price < s_break:
                                target_pos_value = -1  # 做空
                                open_position_price = quote.last_price
        except BacktestFinished:
            # 测试
            trade_log = json.dumps(sim.trade_log)
            # print(trade_log)  # 用于生成测试用例except数据
            excepted_trade_log = '{"2020-06-01": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 10000000.0, "available": 10000000.0, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {}}, "2020-06-02": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 10000000.0, "available": 10000000.0, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {}}, "2020-06-03": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e", "trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|1", "exchange_trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|1", "exchange_id": "SHFE", "instrument_id": "au2012", "direction": "SELL", "offset": "OPEN", "price": 392.88, "volume": 1, "trade_date_time": 1591115339999999000, "commission": 10.0}], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 9999830.0, "available": 9991051.2, "float_profit": -160.000000000025, "position_profit": -160.000000000025, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 10.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008778949242137116, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 1, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 1, "volume_short_his": 0, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": NaN, "position_price_short": 392.88, "position_cost_long": 0.0, "position_cost_short": 392880.0, "float_profit_long": 0.0, "float_profit_short": -160.000000000025, "float_profit": -160.000000000025, "position_profit_long": 0.0, "position_profit_short": -160.000000000025, "position_profit": -160.000000000025, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 393.04, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-04": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9999830.0, "static_balance": 9999830.0, "balance": 10003130.0, "available": 9994351.2, "float_profit": 3139.9999999999836, "position_profit": 3300.0000000000095, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008776053095381145, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 393.04, "position_price_short": 393.04, "position_cost_long": 0.0, "position_cost_short": 392720.0, "float_profit_long": 0.0, "float_profit_short": 3139.9999999999836, "float_profit": 3139.9999999999836, "position_profit_long": 0.0, "position_profit_short": 3300.0000000000095, "position_profit": 3300.0000000000095, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 389.74, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-05": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10003130.0, "static_balance": 10003130.0, "balance": 10002690.0, "available": 9993911.2, "float_profit": 2699.999999999986, "position_profit": -439.9999999999977, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008776439137871912, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 389.74, "position_price_short": 389.74, "position_cost_long": 0.0, "position_cost_short": 396020.0, "float_profit_long": 0.0, "float_profit_short": 2699.999999999986, "float_profit": 2699.999999999986, "position_profit_long": 0.0, "position_profit_short": -439.9999999999977, "position_profit": -439.9999999999977, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 390.18, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-08": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10002690.0, "static_balance": 10002690.0, "balance": 10006590.0, "available": 9997811.2, "float_profit": 6599.9999999999945, "position_profit": 3900.0000000000346, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008773018580755282, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 390.18, "position_price_short": 390.18, "position_cost_long": 0.0, "position_cost_short": 395580.0, "float_profit_long": 0.0, "float_profit_short": 6599.9999999999945, "float_profit": 6599.9999999999945, "position_profit_long": 0.0, "position_profit_short": 3900.0000000000346, "position_profit": 3900.0000000000346, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 386.28, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-09": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10006590.0, "static_balance": 10006590.0, "balance": 10006570.0, "available": 9997791.2, "float_profit": 6579.999999999964, "position_profit": -20.000000000038654, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008773036115272265, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 386.28, "position_price_short": 386.28, "position_cost_long": 0.0, "position_cost_short": 399480.0, "float_profit_long": 0.0, "float_profit_short": 6579.999999999964, "float_profit": 6579.999999999964, "position_profit_long": 0.0, "position_profit_short": -20.000000000038654, "position_profit": -20.000000000038654, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 386.3, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-10": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10006570.0, "static_balance": 10006570.0, "balance": 10002470.0, "available": 9993691.2, "float_profit": 2479.999999999998, "position_profit": -4099.999999999961, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008776632171853552, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 386.3, "position_price_short": 386.3, "position_cost_long": 0.0, "position_cost_short": 399459.99999999994, "float_profit_long": 0.0, "float_profit_short": 2479.999999999998, "float_profit": 2479.999999999998, "position_profit_long": 0.0, "position_profit_short": -4099.999999999961, "position_profit": -4099.999999999961, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 390.4, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-11": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10002470.0, "static_balance": 10002470.0, "balance": 9999170.0, "available": 9990391.2, "float_profit": -820.0000000000132, "position_profit": -3300.0000000000114, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008779528700882173, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 390.4, "position_price_short": 390.4, "position_cost_long": 0.0, "position_cost_short": 395360.0, "float_profit_long": 0.0, "float_profit_short": -820.0000000000132, "float_profit": -820.0000000000132, "position_profit_long": 0.0, "position_profit_short": -3300.0000000000114, "position_profit": -3300.0000000000114, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 393.7, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-12": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9999170.0, "static_balance": 9999170.0, "balance": 9999250.0, "available": 9990471.2, "float_profit": -740.0000000000291, "position_profit": 79.99999999998408, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008779458459384454, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 393.7, "position_price_short": 393.7, "position_cost_long": 0.0, "position_cost_short": 392060.0, "float_profit_long": 0.0, "float_profit_short": -740.0000000000291, "float_profit": -740.0000000000291, "position_profit_long": 0.0, "position_profit_short": 79.99999999998408, "position_profit": 79.99999999998408, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 393.62, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-15": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9999250.0, "static_balance": 9999250.0, "balance": 10000330.0, "available": 9991551.2, "float_profit": 339.999999999955, "position_profit": 1079.999999999984, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008778510309159797, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 393.62, "position_price_short": 393.62, "position_cost_long": 0.0, "position_cost_short": 392140.0, "float_profit_long": 0.0, "float_profit_short": 339.999999999955, "float_profit": 339.999999999955, "position_profit_long": 0.0, "position_profit_short": 1079.999999999984, "position_profit": 1079.999999999984, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 392.54, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-16": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000330.0, "static_balance": 10000330.0, "balance": 10000470.0, "available": 9991691.2, "float_profit": 479.99999999999727, "position_profit": 140.00000000004366, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008778387415791457, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 392.54, "position_price_short": 392.54, "position_cost_long": 0.0, "position_cost_short": 393219.99999999994, "float_profit_long": 0.0, "float_profit_short": 479.99999999999727, "float_profit": 479.99999999999727, "position_profit_long": 0.0, "position_profit_short": 140.00000000004366, "position_profit": 140.00000000004366, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 392.4, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-17": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000470.0, "static_balance": 10000470.0, "balance": 10000010.0, "available": 9991231.2, "float_profit": 19.99999999996089, "position_profit": -460.0000000000364, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008778791221208778, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 392.4, "position_price_short": 392.4, "position_cost_long": 0.0, "position_cost_short": 393360.0, "float_profit_long": 0.0, "float_profit_short": 19.99999999996089, "float_profit": 19.99999999996089, "position_profit_long": 0.0, "position_profit_short": -460.0000000000364, "position_profit": -460.0000000000364, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 392.86, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-18": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000010.0, "static_balance": 10000010.0, "balance": 10000690.0, "available": 9991911.2, "float_profit": 699.9999999999677, "position_profit": 680.0000000000068, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008778194304592983, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 392.86, "position_price_short": 392.86, "position_cost_long": 0.0, "position_cost_short": 392899.99999999994, "float_profit_long": 0.0, "float_profit_short": 699.9999999999677, "float_profit": 699.9999999999677, "position_profit_long": 0.0, "position_profit_short": 680.0000000000068, "position_profit": 680.0000000000068, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 392.18, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-19": {"trades": [], "account": {"currency": "CNY", "pre_balance": 10000690.0, "static_balance": 10000690.0, "balance": 9999930.0, "available": 9991151.2, "float_profit": -60.00000000002319, "position_profit": -759.9999999999909, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008778861452030163, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 392.18, "position_price_short": 392.18, "position_cost_long": 0.0, "position_cost_short": 393579.99999999994, "float_profit_long": 0.0, "float_profit_short": -60.00000000002319, "float_profit": -60.00000000002319, "position_profit_long": 0.0, "position_profit_short": -759.9999999999909, "position_profit": -759.9999999999909, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 392.94, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-22": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9999930.0, "static_balance": 9999930.0, "balance": 9995370.0, "available": 9986591.2, "float_profit": -4620.000000000018, "position_profit": -4559.9999999999945, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008782866467174301, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 392.94, "position_price_short": 392.94, "position_cost_long": 0.0, "position_cost_short": 392820.0, "float_profit_long": 0.0, "float_profit_short": -4620.000000000018, "float_profit": -4620.000000000018, "position_profit_long": 0.0, "position_profit_short": -4559.9999999999945, "position_profit": -4559.9999999999945, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 397.5, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-23": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9995370.0, "static_balance": 9995370.0, "balance": 9995910.0, "available": 9987131.2, "float_profit": -4080.000000000005, "position_profit": 540.0000000000205, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008782391998327315, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 397.5, "position_price_short": 397.5, "position_cost_long": 0.0, "position_cost_short": 388260.0, "float_profit_long": 0.0, "float_profit_short": -4080.000000000005, "float_profit": -4080.000000000005, "position_profit_long": 0.0, "position_profit_short": 540.0000000000205, "position_profit": 540.0000000000205, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 396.96, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-24": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9995910.0, "static_balance": 9995910.0, "balance": 9992810.0, "available": 9984031.2, "float_profit": -7180.000000000029, "position_profit": -3100.0000000000227, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000878511649876261, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 396.96, "position_price_short": 396.96, "position_cost_long": 0.0, "position_cost_short": 388800.0, "float_profit_long": 0.0, "float_profit_short": -7180.000000000029, "float_profit": -7180.000000000029, "position_profit_long": 0.0, "position_profit_short": -3100.0000000000227, "position_profit": -3100.0000000000227, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 400.06, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-29": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9992810.0, "static_balance": 9992810.0, "balance": 9992730.0, "available": 9983951.2, "float_profit": -7260.000000000016, "position_profit": -79.99999999998408, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000878518683082601, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 400.06, "position_price_short": 400.06, "position_cost_long": 0.0, "position_cost_short": 385700.0, "float_profit_long": 0.0, "float_profit_short": -7260.000000000016, "float_profit": -7260.000000000016, "position_profit_long": 0.0, "position_profit_short": -79.99999999998408, "position_profit": -79.99999999998408, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 400.14, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-30": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9992730.0, "static_balance": 9992730.0, "balance": 9992150.0, "available": 9983371.2, "float_profit": -7840.000000000053, "position_profit": -580.0000000000409, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008785696771965993, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 1, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 1, "volume_short": 1, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 392.88, "open_cost_long": 0.0, "open_cost_short": 392880.0, "position_price_long": 400.14, "position_price_short": 400.14, "position_cost_long": 0.0, "position_cost_short": 385620.0, "float_profit_long": 0.0, "float_profit_short": -7840.000000000053, "float_profit": -7840.000000000053, "position_profit_long": 0.0, "position_profit_short": -580.0000000000409, "position_profit": -580.0000000000409, "margin_long": 0.0, "margin_short": 8778.8, "margin": 8778.8, "last_price": 400.72, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-01": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a", "trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|1", "exchange_trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|1", "exchange_id": "SHFE", "instrument_id": "au2012", "direction": "BUY", "offset": "CLOSE", "price": 403.21999999999997, "volume": 1, "trade_date_time": 1593581400000000000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b", "trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|1", "exchange_trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|1", "exchange_id": "SHFE", "instrument_id": "au2012", "direction": "BUY", "offset": "OPEN", "price": 403.21999999999997, "volume": 1, "trade_date_time": 1593581400000000000, "commission": 10.0}], "account": {"currency": "CNY", "pre_balance": 9992150.0, "static_balance": 9992150.0, "balance": 9989510.0, "available": 9980731.2, "float_profit": -119.9999999999477, "position_profit": -119.9999999999477, "close_profit": -2499.999999999943, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 20.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.000878801863154449, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 0, "pos_long_today": 1, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 1, "volume_long_his": 0, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 403.2199999999999, "position_price_short": NaN, "position_cost_long": 403219.99999999994, "position_cost_short": 0.0, "float_profit_long": -119.9999999999477, "float_profit_short": 0.0, "float_profit": -119.9999999999477, "position_profit_long": -119.9999999999477, "position_profit_short": 0.0, "position_profit": -119.9999999999477, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 403.1, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-02": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9989510.0, "static_balance": 9989510.0, "balance": 9985470.0, "available": 9976691.2, "float_profit": -4159.999999999969, "position_profit": -4040.000000000019, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008791574157250484, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 403.1, "position_price_short": 403.1, "position_cost_long": 403100.0, "position_cost_short": 0.0, "float_profit_long": -4159.999999999969, "float_profit_short": 0.0, "float_profit": -4159.999999999969, "position_profit_long": -4040.000000000019, "position_profit_short": 0.0, "position_profit": -4040.000000000019, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 399.06, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-03": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9985470.0, "static_balance": 9985470.0, "balance": 9986950.0, "available": 9978171.2, "float_profit": -2679.9999999999477, "position_profit": 1480.0000000000182, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008790271304051787, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 399.06, "position_price_short": 399.06, "position_cost_long": 399060.0, "position_cost_short": 0.0, "float_profit_long": -2679.9999999999477, "float_profit_short": 0.0, "float_profit": -2679.9999999999477, "position_profit_long": 1480.0000000000182, "position_profit_short": 0.0, "position_profit": 1480.0000000000182, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 400.54, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-06": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9986950.0, "static_balance": 9986950.0, "balance": 9986070.0, "available": 9977291.2, "float_profit": -3559.999999999943, "position_profit": -879.9999999999955, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008791045926976278, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 400.54, "position_price_short": 400.54, "position_cost_long": 400540.0, "position_cost_short": 0.0, "float_profit_long": -3559.999999999943, "float_profit_short": 0.0, "float_profit": -3559.999999999943, "position_profit_long": -879.9999999999955, "position_profit_short": 0.0, "position_profit": -879.9999999999955, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 399.66, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-07": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9986070.0, "static_balance": 9986070.0, "balance": 9986730.0, "available": 9977951.2, "float_profit": -2899.999999999975, "position_profit": 659.9999999999682, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008790464946984648, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 399.66, "position_price_short": 399.66, "position_cost_long": 399660.0, "position_cost_short": 0.0, "float_profit_long": -2899.999999999975, "float_profit_short": 0.0, "float_profit": -2899.999999999975, "position_profit_long": 659.9999999999682, "position_profit_short": 0.0, "position_profit": 659.9999999999682, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 400.32, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-08": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9986730.0, "static_balance": 9986730.0, "balance": 9988770.0, "available": 9979991.2, "float_profit": -859.9999999999541, "position_profit": 2040.0000000000205, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008788669676046199, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 400.32, "position_price_short": 400.32, "position_cost_long": 400319.99999999994, "position_cost_short": 0.0, "float_profit_long": -859.9999999999541, "float_profit_short": 0.0, "float_profit": -859.9999999999541, "position_profit_long": 2040.0000000000205, "position_profit_short": 0.0, "position_profit": 2040.0000000000205, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 402.36, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-09": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9988770.0, "static_balance": 9988770.0, "balance": 9991350.0, "available": 9982571.2, "float_profit": 1720.00000000003, "position_profit": 2579.999999999984, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008786400236204317, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 402.36, "position_price_short": 402.36, "position_cost_long": 402360.0, "position_cost_short": 0.0, "float_profit_long": 1720.00000000003, "float_profit_short": 0.0, "float_profit": 1720.00000000003, "position_profit_long": 2579.999999999984, "position_profit_short": 0.0, "position_profit": 2579.999999999984, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 404.94, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-10": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9991350.0, "static_balance": 9991350.0, "balance": 9988290.0, "available": 9979511.2, "float_profit": -1339.9999999999723, "position_profit": -3060.0000000000023, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008789092026763339, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 404.94, "position_price_short": 404.94, "position_cost_long": 404940.0, "position_cost_short": 0.0, "float_profit_long": -1339.9999999999723, "float_profit_short": 0.0, "float_profit": -1339.9999999999723, "position_profit_long": -3060.0000000000023, "position_profit_short": 0.0, "position_profit": -3060.0000000000023, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 401.88, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-13": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9988290.0, "static_balance": 9988290.0, "balance": 9990150.0, "available": 9981371.2, "float_profit": 520.0000000000414, "position_profit": 1860.0000000000136, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008787455643809152, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 401.88, "position_price_short": 401.88, "position_cost_long": 401879.99999999994, "position_cost_short": 0.0, "float_profit_long": 520.0000000000414, "float_profit_short": 0.0, "float_profit": 520.0000000000414, "position_profit_long": 1860.0000000000136, "position_profit_short": 0.0, "position_profit": 1860.0000000000136, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 403.74, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-14": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9990150.0, "static_balance": 9990150.0, "balance": 9989430.0, "available": 9980651.2, "float_profit": -199.9999999999859, "position_profit": -720.0000000000273, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008788089010083658, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 403.74, "position_price_short": 403.74, "position_cost_long": 403740.0, "position_cost_short": 0.0, "float_profit_long": -199.9999999999859, "float_profit_short": 0.0, "float_profit": -199.9999999999859, "position_profit_long": -720.0000000000273, "position_profit_short": 0.0, "position_profit": -720.0000000000273, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 403.02, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-15": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9989430.0, "static_balance": 9989430.0, "balance": 9990030.0, "available": 9981251.2, "float_profit": 400.00000000003683, "position_profit": 600.0000000000227, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008787561198514919, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 403.02, "position_price_short": 403.02, "position_cost_long": 403019.99999999994, "position_cost_short": 0.0, "float_profit_long": 400.00000000003683, "float_profit_short": 0.0, "float_profit": 400.00000000003683, "position_profit_long": 600.0000000000227, "position_profit_short": 0.0, "position_profit": 600.0000000000227, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 403.62, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-16": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9990030.0, "static_balance": 9990030.0, "balance": 9988530.0, "available": 9979751.2, "float_profit": -1099.9999999999632, "position_profit": -1500.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008788880846330741, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 403.62, "position_price_short": 403.62, "position_cost_long": 403620.0, "position_cost_short": 0.0, "float_profit_long": -1099.9999999999632, "float_profit_short": 0.0, "float_profit": -1099.9999999999632, "position_profit_long": -1500.0, "position_profit_short": 0.0, "position_profit": -1500.0, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 402.12, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-17": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9988530.0, "static_balance": 9988530.0, "balance": 9986170.0, "available": 9977391.2, "float_profit": -3459.999999999977, "position_profit": -2360.0000000000136, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008790957894768464, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 402.12, "position_price_short": 402.12, "position_cost_long": 402120.0, "position_cost_short": 0.0, "float_profit_long": -3459.999999999977, "float_profit_short": 0.0, "float_profit": -3459.999999999977, "position_profit_long": -2360.0000000000136, "position_profit_short": 0.0, "position_profit": -2360.0000000000136, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 399.76, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-07-20": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9986170.0, "static_balance": 9986170.0, "balance": 9988190.0, "available": 9979411.2, "float_profit": -1439.999999999995, "position_profit": 2019.9999999999818, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 8778.8, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0008789180021605515, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.au2012": {"exchange_id": "SHFE", "instrument_id": "au2012", "pos_long_his": 1, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 1, "volume_long": 1, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 403.2199999999999, "open_price_short": NaN, "open_cost_long": 403219.99999999994, "open_cost_short": 0.0, "position_price_long": 399.76, "position_price_short": 399.76, "position_cost_long": 399759.99999999994, "position_cost_short": 0.0, "float_profit_long": -1439.999999999995, "float_profit_short": 0.0, "float_profit": -1439.999999999995, "position_profit_long": 2019.9999999999818, "position_profit_short": 0.0, "position_profit": 2019.9999999999818, "margin_long": 8778.8, "margin_short": 0.0, "margin": 8778.8, "last_price": 401.78, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}}'
            self.assertEqual(excepted_trade_log, trade_log)

    # @unittest.skip("无条件跳过: test_fairy_four_price")
    def test_fairy_four_price(self):
        '''
        设定：
            回测周期: 2019,4,1-2019,4,20
            合约代码: SHFE.cu1905
            合约文件: "http://127.0.0.1:5000/t/md/symbols/2019-06-10.json"
        '''
        # 预设服务器端响应
        dir_path = os.path.dirname(os.path.realpath(__file__))
        self.mock.run(os.path.join(dir_path, "examples_log_file", "test_example_fairy_four_price.script.lzma"))
        # # 测试
        symbol = "SHFE.cu2012"  # 合约代码
        close_hour, close_minute = 14, 50  # 平仓时间
        utils.RD = random.Random(4)
        sim = TqSim()
        api = TqApi(sim, backtest=TqBacktest(start_dt=date(2020, 6, 1), end_dt=date(2020, 6, 20)),
                    auth="tianqin,tianqin", _td_url=self.td_url, _md_url=self.md_url)
        quote = api.get_quote(symbol)  # 获取指定合约的盘口行情
        klines = api.get_kline_serial(symbol, 24 * 60 * 60)  # 获取日线
        position = api.get_position(symbol)  # 持仓信息
        target_pos = TargetPosTask(api, symbol)  # 目标持仓

        top_rail = klines.high.iloc[-2]  # 上轨: 昨日高点
        bottom_rail = klines.low.iloc[-2]  # 下轨: 昨日低点

        try:
            with closing(api):
                while True:
                    api.wait_update()
                    if api.is_changing(klines.iloc[-1], "datetime"):  # 如果产生一根新日线 (即到达下一个交易日): 重新获取上下轨
                        top_rail = klines.high.iloc[-2]
                        bottom_rail = klines.low.iloc[-2]

                    if api.is_changing(quote, "last_price"):  # 如果行情最新价发生变化
                        # 开仓突破
                        if quote.last_price > top_rail and position.volume_long == 0:  # 如果价格突破上轨: 买入开仓
                            target_pos.set_target_volume(3)  # 设置目标持仓手数，将指定合约调整到目标头寸
                        elif quote.last_price < bottom_rail and position.volume_short == 0:  # 如果价格跌破下轨: 卖出开仓
                            target_pos.set_target_volume(-3)

                        # 平仓止损: 当价格 向上突破上轨 或 向下突破下轨 后, 再次回破当日开盘价
                        if (quote.highest > top_rail and quote.last_price <= quote.open) or (
                                quote.lowest < bottom_rail and quote.last_price >= quote.open):
                            target_pos.set_target_volume(0)

                    if api.is_changing(quote, "datetime"):
                        now_time = datetime.strptime(quote.datetime, "%Y-%m-%d %H:%M:%S.%f")  # 获取当前的行情时间
                        if now_time.hour == close_hour and now_time.minute >= close_minute:  # 到达平仓时间: 平仓
                            target_pos.set_target_volume(0)
                            top_rail = bottom_rail = float("nan")
        except BacktestFinished:
            # 测试
            trade_log = json.dumps(sim.trade_log)
            # print(trade_log)  # 用于生成测试用例except数据
            excepted_trade_log = '{"2020-06-01": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e", "trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|3", "exchange_trade_id": "PYSDK_target_c79d679346d4ac7a5c3902b38963dc6e|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 44060.0, "volume": 3, "trade_date_time": 1590757319999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a", "trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|3", "exchange_trade_id": "PYSDK_target_43000de01b2ed40ed3addccb2c33be0a|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 44480.0, "volume": 3, "trade_date_time": 1590994200000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 10006222.96, "available": 10006222.96, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 6300.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": NaN, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 44500.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-02": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b", "trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|3", "exchange_trade_id": "PYSDK_target_06905269ed6f0b09f165c8ce36e2f24b|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 44610.0, "volume": 3, "trade_date_time": 1591025039999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad", "trade_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad|3", "exchange_trade_id": "PYSDK_target_42a00403ce80c4b0a4042bb3d4341aad|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 44470.0, "volume": 3, "trade_date_time": 1591080600000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10006222.96, "static_balance": 10006222.96, "balance": 10004045.920000002, "available": 10004045.920000002, "float_profit": 0.0, "position_profit": 0.0, "close_profit": -2100.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 44500.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 44510.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-03": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_2a3187853184ff27459142deccea2645", "trade_id": "PYSDK_target_2a3187853184ff27459142deccea2645|3", "exchange_trade_id": "PYSDK_target_2a3187853184ff27459142deccea2645|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 44520.0, "volume": 3, "trade_date_time": 1591102800000000000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_de08caa1a08179104a25e4664f5253a0", "trade_id": "PYSDK_target_de08caa1a08179104a25e4664f5253a0|3", "exchange_trade_id": "PYSDK_target_de08caa1a08179104a25e4664f5253a0|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 44700.0, "volume": 3, "trade_date_time": 1591167000000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10004045.920000002, "static_balance": 10004045.920000002, "balance": 10006668.880000003, "available": 10006668.880000003, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 2700.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 44510.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 44740.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-04": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_d93936e1daca3c06f5ff0c03bb5d7385", "trade_id": "PYSDK_target_d93936e1daca3c06f5ff0c03bb5d7385|3", "exchange_trade_id": "PYSDK_target_d93936e1daca3c06f5ff0c03bb5d7385|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "OPEN", "price": 44640.0, "volume": 3, "trade_date_time": 1591232639999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d8441b5616332aca5f552773e14b0190", "trade_id": "PYSDK_target_d8441b5616332aca5f552773e14b0190|3", "exchange_trade_id": "PYSDK_target_d8441b5616332aca5f552773e14b0190|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "CLOSETODAY", "price": 44700.0, "volume": 3, "trade_date_time": 1591253400000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10006668.880000003, "static_balance": 10006668.880000003, "balance": 10005691.840000004, "available": 10005691.840000004, "float_profit": 0.0, "position_profit": 0.0, "close_profit": -900.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": 44740.0, "position_price_short": NaN, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 44690.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-05": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_634f806fabf4a07c566002249b191bf4", "trade_id": "PYSDK_target_634f806fabf4a07c566002249b191bf4|3", "exchange_trade_id": "PYSDK_target_634f806fabf4a07c566002249b191bf4|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 45000.0, "volume": 3, "trade_date_time": 1591336199999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_3f5082492d83a8233fb62d2c81862fc9", "trade_id": "PYSDK_target_3f5082492d83a8233fb62d2c81862fc9|3", "exchange_trade_id": "PYSDK_target_3f5082492d83a8233fb62d2c81862fc9|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 45100.0, "volume": 3, "trade_date_time": 1591339800000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10005691.840000004, "static_balance": 10005691.840000004, "balance": 10007114.800000004, "available": 10007114.800000004, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 1500.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 44690.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 45180.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-08": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_f1cfd99216df648647adec26793d0e45", "trade_id": "PYSDK_target_f1cfd99216df648647adec26793d0e45|3", "exchange_trade_id": "PYSDK_target_f1cfd99216df648647adec26793d0e45|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 45190.0, "volume": 3, "trade_date_time": 1591362000000000000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f1347e0cdd905ecfd160c5d0ef412ed6", "trade_id": "PYSDK_target_f1347e0cdd905ecfd160c5d0ef412ed6|3", "exchange_trade_id": "PYSDK_target_f1347e0cdd905ecfd160c5d0ef412ed6|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 45810.0, "volume": 3, "trade_date_time": 1591599000000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10007114.800000004, "static_balance": 10007114.800000004, "balance": 10016337.760000005, "available": 10016337.760000005, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 9300.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 45180.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 45750.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-09": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_01d89a024cdce7a6d7288ff68c320f89", "trade_id": "PYSDK_target_01d89a024cdce7a6d7288ff68c320f89|3", "exchange_trade_id": "PYSDK_target_01d89a024cdce7a6d7288ff68c320f89|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 45760.0, "volume": 3, "trade_date_time": 1591621200000000000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_b474c7e89286a1754abcb06ae8abb93f", "trade_id": "PYSDK_target_b474c7e89286a1754abcb06ae8abb93f|3", "exchange_trade_id": "PYSDK_target_b474c7e89286a1754abcb06ae8abb93f|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 46130.0, "volume": 3, "trade_date_time": 1591685400000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10016337.760000005, "static_balance": 10016337.760000005, "balance": 10021810.720000006, "available": 10021810.720000006, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 5550.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 45750.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46200.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-10": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_c3e4a892d9196ada4fcfa583e1df8af9", "trade_id": "PYSDK_target_c3e4a892d9196ada4fcfa583e1df8af9|3", "exchange_trade_id": "PYSDK_target_c3e4a892d9196ada4fcfa583e1df8af9|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 46420.0, "volume": 3, "trade_date_time": 1591713299999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_6c79a3de69f85e3131f3b9238224b122", "trade_id": "PYSDK_target_6c79a3de69f85e3131f3b9238224b122|3", "exchange_trade_id": "PYSDK_target_6c79a3de69f85e3131f3b9238224b122|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 46650.0, "volume": 3, "trade_date_time": 1591771800000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10021810.720000006, "static_balance": 10021810.720000006, "balance": 10025183.680000007, "available": 10025183.680000007, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 3450.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 46200.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46800.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-11": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_738d243a6e58d5ca49c7b59b995253fd", "trade_id": "PYSDK_target_738d243a6e58d5ca49c7b59b995253fd|3", "exchange_trade_id": "PYSDK_target_738d243a6e58d5ca49c7b59b995253fd|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 47260.0, "volume": 3, "trade_date_time": 1591794000000000000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_4278c2614e1bcb383bb4a570294c4ea3", "trade_id": "PYSDK_target_4278c2614e1bcb383bb4a570294c4ea3|3", "exchange_trade_id": "PYSDK_target_4278c2614e1bcb383bb4a570294c4ea3|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 46830.0, "volume": 3, "trade_date_time": 1591858200000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10025183.680000007, "static_balance": 10025183.680000007, "balance": 10018656.640000008, "available": 10018656.640000008, "float_profit": 0.0, "position_profit": 0.0, "close_profit": -6450.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 46800.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46900.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-12": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_14c15c910b11ad28cc21ce88d0060cc5", "trade_id": "PYSDK_target_14c15c910b11ad28cc21ce88d0060cc5|3", "exchange_trade_id": "PYSDK_target_14c15c910b11ad28cc21ce88d0060cc5|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "OPEN", "price": 46540.0, "volume": 3, "trade_date_time": 1591880519999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_ff5a52f1a05885ac7671863c0bdbc23a", "trade_id": "PYSDK_target_ff5a52f1a05885ac7671863c0bdbc23a|3", "exchange_trade_id": "PYSDK_target_ff5a52f1a05885ac7671863c0bdbc23a|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "CLOSETODAY", "price": 46800.0, "volume": 3, "trade_date_time": 1591944600000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10018656.640000008, "static_balance": 10018656.640000008, "balance": 10014679.600000009, "available": 10014679.600000009, "float_profit": 0.0, "position_profit": 0.0, "close_profit": -3900.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": 46900.0, "position_price_short": NaN, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46680.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-15": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_a5e333cb88dcf94384d4cd1f47ca7883", "trade_id": "PYSDK_target_a5e333cb88dcf94384d4cd1f47ca7883|3", "exchange_trade_id": "PYSDK_target_a5e333cb88dcf94384d4cd1f47ca7883|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 47040.0, "volume": 3, "trade_date_time": 1591969259999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2522d53857c49391b36cc9aa78a330a1", "trade_id": "PYSDK_target_2522d53857c49391b36cc9aa78a330a1|3", "exchange_trade_id": "PYSDK_target_2522d53857c49391b36cc9aa78a330a1|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 45980.0, "volume": 3, "trade_date_time": 1592203439999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_11021c9e32111ac1ac7cc4a4ff4dab10", "trade_id": "PYSDK_target_11021c9e32111ac1ac7cc4a4ff4dab10|3", "exchange_trade_id": "PYSDK_target_11021c9e32111ac1ac7cc4a4ff4dab10|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "OPEN", "price": 45980.0, "volume": 3, "trade_date_time": 1592203439999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_a2909cb633e238b4e9dd38b869ace913", "trade_id": "PYSDK_target_a2909cb633e238b4e9dd38b869ace913|3", "exchange_trade_id": "PYSDK_target_a2909cb633e238b4e9dd38b869ace913|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "CLOSETODAY", "price": 46020.0, "volume": 3, "trade_date_time": 1592203800000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 10014679.600000009, "static_balance": 10014679.600000009, "balance": 9998025.52000001, "available": 9998025.52000001, "float_profit": 0.0, "position_profit": 0.0, "close_profit": -16500.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 154.07999999999998, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": NaN, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 45970.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-16": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_2f0733c846bbe9e870ef55b1a1f65507", "trade_id": "PYSDK_target_2f0733c846bbe9e870ef55b1a1f65507|3", "exchange_trade_id": "PYSDK_target_2f0733c846bbe9e870ef55b1a1f65507|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "OPEN", "price": 45920.0, "volume": 3, "trade_date_time": 1592226000000000000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_96b98b5fbf37a2be6f98bca35b17b966", "trade_id": "PYSDK_target_96b98b5fbf37a2be6f98bca35b17b966|3", "exchange_trade_id": "PYSDK_target_96b98b5fbf37a2be6f98bca35b17b966|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "CLOSETODAY", "price": 46840.0, "volume": 3, "trade_date_time": 1592290200000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 9998025.52000001, "static_balance": 9998025.52000001, "balance": 9984148.480000012, "available": 9984148.480000012, "float_profit": 0.0, "position_profit": 0.0, "close_profit": -13800.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": 45970.0, "position_price_short": NaN, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46850.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-17": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9984148.480000012, "static_balance": 9984148.480000012, "balance": 9984148.480000012, "available": 9984148.480000012, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": 46850.0, "position_price_short": 46850.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46430.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-18": {"trades": [], "account": {"currency": "CNY", "pre_balance": 9984148.480000012, "static_balance": 9984148.480000012, "balance": 9984148.480000012, "available": 9984148.480000012, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 0.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": 46430.0, "position_price_short": 46430.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 46540.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-19": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_32decd6b8efbc170a26a25c852175b7a", "trade_id": "PYSDK_target_32decd6b8efbc170a26a25c852175b7a|3", "exchange_trade_id": "PYSDK_target_32decd6b8efbc170a26a25c852175b7a|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 46670.0, "volume": 3, "trade_date_time": 1592489399999999000, "commission": 38.519999999999996}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d6e4a51519d9c9cc52d32377e78131c1", "trade_id": "PYSDK_target_d6e4a51519d9c9cc52d32377e78131c1|3", "exchange_trade_id": "PYSDK_target_d6e4a51519d9c9cc52d32377e78131c1|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "SELL", "offset": "CLOSETODAY", "price": 47280.0, "volume": 3, "trade_date_time": 1592549400000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 9984148.480000012, "static_balance": 9984148.480000012, "balance": 9993221.440000013, "available": 9993221.440000013, "float_profit": 0.0, "position_profit": 0.0, "close_profit": 9150.0, "frozen_margin": 0.0, "margin": 0.0, "frozen_commission": 0.0, "commission": 77.03999999999999, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": NaN, "open_cost_long": 0.0, "open_cost_short": 0.0, "position_price_long": NaN, "position_price_short": 46540.0, "position_cost_long": 0.0, "position_cost_short": 0.0, "float_profit_long": 0.0, "float_profit_short": 0.0, "float_profit": 0.0, "position_profit_long": 0.0, "position_profit_short": 0.0, "position_profit": 0.0, "margin_long": 0.0, "margin_short": 0.0, "margin": 0.0, "last_price": 47300.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}, "2020-06-22": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_target_4708d9893a973000b54a23020fc5b043", "trade_id": "PYSDK_target_4708d9893a973000b54a23020fc5b043|3", "exchange_trade_id": "PYSDK_target_4708d9893a973000b54a23020fc5b043|3", "exchange_id": "SHFE", "instrument_id": "cu2012", "direction": "BUY", "offset": "OPEN", "price": 47560.0, "volume": 3, "trade_date_time": 1592571600000000000, "commission": 38.519999999999996}], "account": {"currency": "CNY", "pre_balance": 9993221.440000013, "static_balance": 9993221.440000013, "balance": 9984182.920000013, "available": 9930254.920000013, "float_profit": -9000.0, "position_profit": -9000.0, "close_profit": 0.0, "frozen_margin": 0.0, "margin": 53928.0, "frozen_commission": 0.0, "commission": 38.519999999999996, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.005401343347984247, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"SHFE.cu2012": {"exchange_id": "SHFE", "instrument_id": "cu2012", "pos_long_his": 0, "pos_long_today": 3, "pos_short_his": 0, "pos_short_today": 0, "volume_long_today": 3, "volume_long_his": 0, "volume_long": 3, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 0, "volume_short_his": 0, "volume_short": 0, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": 47560.0, "open_price_short": NaN, "open_cost_long": 713400.0, "open_cost_short": 0.0, "position_price_long": 47560.0, "position_price_short": 47300.0, "position_cost_long": 713400.0, "position_cost_short": 0.0, "float_profit_long": -9000.0, "float_profit_short": 0.0, "float_profit": -9000.0, "position_profit_long": -9000.0, "position_profit_short": 0.0, "position_profit": -9000.0, "margin_long": 53928.0, "margin_short": 0.0, "margin": 53928.0, "last_price": 46960.0, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0}}}}'
            self.assertEqual(excepted_trade_log, trade_log)
